# -*- coding: utf-8 -*-
# Generated by Django 1.11.10 on 2018-02-28 15:53
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('uk_geo_utils', '0003_auto_20171030_1504'),
    ]

    operations = [
        migrations.RenameField(
            model_name='onspd',
            old_name='hro',
            new_name='nhser',
        ),
        migrations.RenameField(
            model_name='onspd',
            old_name='gor',
            new_name='rgn',
        ),
        migrations.RemoveField(
            model_name='onspd',
            name='cened',
        ),
        migrations.RemoveField(
            model_name='onspd',
            name='edind',
        ),
        migrations.RemoveField(
            model_name='onspd',
            name='lea',
        ),
        migrations.RemoveField(
            model_name='onspd',
            name='oldha',
        ),
        migrations.RemoveField(
            model_name='onspd',
            name='oldpct',
        ),
        migrations.RemoveField(
            model_name='onspd',
            name='oshaprev',
        ),
        migrations.RemoveField(
            model_name='onspd',
            name='psed',
        ),
        migrations.RemoveField(
            model_name='onspd',
            name='streg',
        ),
        migrations.RemoveField(
            model_name='onspd',
            name='ward98',
        ),
        migrations.RemoveField(
            model_name='onspd',
            name='wardc91',
        ),
        migrations.RemoveField(
            model_name='onspd',
            name='wardo91',
        ),
        migrations.AddField(
            model_name='onspd',
            name='ced',
            field=models.CharField(blank=True, max_length=9),
        ),
    ]



import csv
import os
import glob
from uk_geo_utils.helpers import AddressFormatter
from django.core.management.base import BaseCommand


class Command(BaseCommand):

    def add_arguments(self, parser):
        parser.add_argument(
            'ab_path',
            help='The path to the folder containing the AddressBase CSVs'
        )

    def handle(self, *args, **kwargs):
        self.fieldnames = [
            'UPRN',
            'OS_ADDRESS_TOID',
            'UDPRN',
            'ORGANISATION_NAME',
            'DEPARTMENT_NAME',
            'PO_BOX_NUMBER',
            'SUB_BUILDING_NAME',
            'BUILDING_NAME',
            'BUILDING_NUMBER',
            'DEPENDENT_THOROUGHFARE',
            'THOROUGHFARE',
            'POST_TOWN',
            'DOUBLE_DEPENDENT_LOCALITY',
            'DEPENDENT_LOCALITY',
            'POSTCODE',
            'POSTCODE_TYPE',
            'X_COORDINATE',
            'Y_COORDINATE',
            'LATITUDE',
            'LONGITUDE',
            'RPC',
            'COUNTRY',
            'CHANGE_TYPE',
            'LA_START_DATE',
            'RM_START_DATE',
            'LAST_UPDATE_DATE',
            'CLASS',
        ]
        self.base_path = os.path.abspath(kwargs['ab_path'])
        out_path = os.path.join(self.base_path, 'addressbase_cleaned.csv')

        files = glob.glob(os.path.join(self.base_path, '*.csv'))
        if not files:
            raise FileNotFoundError('No CSV files found in %s' % (self.base_path))

        with open(out_path, 'w') as out_file:
            for csv_path in files:
                if csv_path.endswith('cleaned.csv'):
                    continue
                self.out_csv = csv.DictWriter(out_file, fieldnames=[
                    'UPRN',
                    'address',
                    'postcode',
                    'location',
                ])
                self.stdout.write(csv_path)
                self.clean_csv(csv_path)
                out_file.flush()

    def line_filer(self, csv_path):
        with open(csv_path) as csv_file:
            for line in csv.DictReader(csv_file, fieldnames=self.fieldnames):
                # Do any filtering we might need to do here
                yield line

    def clean_csv(self, csv_path):
        for line in self.line_filer(csv_path):
            self.out_csv.writerow(self.clean_output_line(line))

    def clean_address(self, line):
        address_fields = [
            'ORGANISATION_NAME',
            'DEPARTMENT_NAME',
            'PO_BOX_NUMBER',
            'SUB_BUILDING_NAME',
            'BUILDING_NAME',
            'BUILDING_NUMBER',
            'DEPENDENT_THOROUGHFARE',
            'THOROUGHFARE',
            'DOUBLE_DEPENDENT_LOCALITY',
            'DEPENDENT_LOCALITY',
            'POST_TOWN'
        ]
        kwargs = {k.lower(): line[k] for k in line if k in address_fields}
        return AddressFormatter(**kwargs).generate_address_label()

    def clean_output_line(self, line):
        data = {}
        data['UPRN'] = line['UPRN']
        data['address'] = self.clean_address(line)
        data['postcode'] = line['POSTCODE']
        data['location'] = "SRID=4326;POINT({} {})".format(
            line['LONGITUDE'],
            line['LATITUDE'],
        )
        return data



import abc
from django.core.exceptions import ObjectDoesNotExist
from uk_geo_utils.helpers import (
    AddressSorter,
    get_address_model,
    get_onspd_model,
    get_onsud_model,
    Postcode,
)


class AddressBaseException(Exception):
    pass

class CodesNotFoundException(AddressBaseException):
    pass

class MultipleCodesException(AddressBaseException):
    pass

class StrictMatchException(AddressBaseException):
    pass

class NorthernIrelandException(ObjectDoesNotExist):
    pass

class AddressBaseNotImportedException(ObjectDoesNotExist):
    pass

class OnsudNotImportedException(ObjectDoesNotExist):
    pass

class OnspdNotImportedException(ObjectDoesNotExist):
    pass


class BaseGeocoder(metaclass=abc.ABCMeta):

    def __init__(self, postcode):
        self.postcode = Postcode(postcode)

    @property
    @abc.abstractmethod
    def centroid(self):
        pass

    @abc.abstractmethod
    def get_code(code_type, uprn=None):
        pass


class AddressBaseGeocoder(BaseGeocoder):

    def __init__(self, postcode):

        self.postcode = Postcode(postcode)
        if self.postcode.territory == 'NI':
            raise NorthernIrelandException('Postcode is in Northern Ireland')

        self.onsud_model = get_onsud_model()
        self.address_model = get_address_model()

        # check the data we need exists
        if not self.address_model.objects.all().exists():
            raise AddressBaseNotImportedException('Address Base table is empty')
        if not self.onsud_model.objects.all().exists():
            raise OnsudNotImportedException('ONSUD table is empty')

        self._addresses = self.address_model.objects.filter(
            postcode=self.postcode.with_space).order_by('uprn')
        if not self._addresses:
            raise self.address_model.DoesNotExist(
                'No addresses found for postcode %s' % (self.postcode))

        self._uprns = self.onsud_model.objects.filter(
            uprn__in=self.uprns).order_by('uprn')

    @property
    def uprns(self):
        return [a.uprn for a in self._addresses]

    @property
    def centroid(self):
        return self._addresses.centroid

    @property
    def addresses(self):
        sorter = AddressSorter(self._addresses)
        return sorter.natural_sort()

    def get_point(self, uprn):
        return self._addresses.get_cached(uprn).location

    def get_code(self, code_type, uprn=None, strict=False):
        # check the code_type field exists on our model
        self.onsud_model._meta.get_field(code_type)

        if uprn:
            self._addresses.get_cached(uprn)
            return getattr(self._uprns.get_cached(uprn), code_type)

        if len(self._uprns) == 0:
            # No records in the ONSUD table were found for the given UPRNs
            # because...reasons
            raise CodesNotFoundException('Found no records in ONSUD for supplied UPRNs')
        if len(self._addresses) != len(self._uprns):
            if strict:
                for uprn in self.uprns:
                    try:
                        self._uprns.get_cached(uprn)
                    except self.onsud_model.DoesNotExist:
                        raise StrictMatchException(
                            'expected UPRN %s not found in ONSUD' % uprn)
            else:
                # if not strict, ignore this condition
                pass

        codes = set([getattr(u, code_type) for u in self._uprns])
        if len(codes) == 1:
            # all the uprns supplied are in the same area
            return list(codes)[0]
        else:
            raise MultipleCodesException(
                "Postcode %s covers UPRNs in more than one '%s' area" %\
                (self.postcode, code_type)
            )


class OnspdGeocoder(BaseGeocoder):

    def __init__(self, postcode):
        self.postcode = Postcode(postcode)
        self.onspd_model = get_onspd_model()

        if not self.onspd_model.objects.all().exists():
            raise OnspdNotImportedException('ONSPD table is empty')

        self.record = self.onspd_model.objects.get(
            pcds=self.postcode.with_space,
            doterm=''
        )

    @property
    def centroid(self):
        return self.record.location

    def get_code(self, code_type):
        return getattr(self.record, code_type)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import django.contrib.gis.db.models.fields


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Address',
            fields=[
                ('uprn', models.CharField(max_length=100, serialize=False, primary_key=True)),
                ('address', models.TextField(blank=True)),
                ('postcode', models.CharField(max_length=15, db_index=True, blank=True)),
                ('location', django.contrib.gis.db.models.fields.PointField(null=True, srid=4326, blank=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Onsud',
            fields=[
                ('uprn', models.CharField(max_length=12, serialize=False, primary_key=True)),
                ('ctry_flag', models.CharField(max_length=1, blank=True)),
                ('cty', models.CharField(max_length=9, blank=True)),
                ('lad', models.CharField(max_length=9, blank=True)),
                ('ward', models.CharField(max_length=9, blank=True)),
                ('hlthau', models.CharField(max_length=9, blank=True)),
                ('ctry', models.CharField(max_length=9, blank=True)),
                ('rgn', models.CharField(max_length=9, blank=True)),
                ('pcon', models.CharField(max_length=9, blank=True)),
                ('eer', models.CharField(max_length=9, blank=True)),
                ('ttwa', models.CharField(max_length=9, blank=True)),
                ('nuts', models.CharField(max_length=9, blank=True)),
                ('park', models.CharField(max_length=9, blank=True)),
                ('oa11', models.CharField(max_length=9, blank=True)),
                ('lsoa11', models.CharField(max_length=9, blank=True)),
                ('msoa11', models.CharField(max_length=9, blank=True)),
                ('parish', models.CharField(max_length=9, blank=True)),
                ('wz11', models.CharField(max_length=9, blank=True)),
                ('ccg', models.CharField(max_length=9, blank=True)),
                ('bua11', models.CharField(max_length=9, blank=True)),
                ('buasd11', models.CharField(max_length=9, blank=True)),
                ('ruc11', models.CharField(max_length=2, blank=True)),
                ('oac11', models.CharField(max_length=3, blank=True)),
                ('lep1', models.CharField(max_length=9, blank=True)),
                ('lep2', models.CharField(max_length=9, blank=True)),
                ('pfa', models.CharField(max_length=9, blank=True)),
                ('imd', models.CharField(max_length=5, blank=True)),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



import os
from django.db import connection
from django.core.management.base import BaseCommand
from uk_geo_utils.helpers import get_address_model


class Command(BaseCommand):

    def add_arguments(self, parser):
        parser.add_argument(
            'cleaned_ab_path',
            help='The path to the folder containing the cleaned AddressBase CSVs'
        )

    def handle(self, *args, **kwargs):
        self.table_name = get_address_model()._meta.db_table

        cleaned_file_path = os.path.abspath(os.path.join(
            kwargs['cleaned_ab_path'],
            "addressbase_cleaned.csv"
        ))

        fp = open(cleaned_file_path, 'r')

        cursor = connection.cursor()
        self.stdout.write("clearing existing data..")
        cursor.execute("TRUNCATE TABLE %s;" % (self.table_name))

        self.stdout.write("importing from %s.." % (cleaned_file_path))
        cursor.copy_expert("""
            COPY %s (UPRN,address,postcode,location)
            FROM STDIN (FORMAT CSV, DELIMITER ',', quote '"');
        """ % (self.table_name), fp)

        self.stdout.write("...done")



import os
import glob
from django.db import connection
from django.core.management.base import BaseCommand
from uk_geo_utils.helpers import get_onsud_model


class Command(BaseCommand):
    """
    To import ONSUD, grab the latest release:
    http://ons.maps.arcgis.com/home/search.html?q=ONS%20Address%20Directory&t=content
    and run
    python manage.py import_onsud /path/to/data
    """

    def add_arguments(self, parser):
        parser.add_argument(
            'path',
            help='Path to the directory containing the ONSUD CSVs'
        )

    def handle(self, *args, **kwargs):
        self.table_name = get_onsud_model()._meta.db_table

        glob_str = os.path.join(kwargs['path'], "*.csv")
        files = glob.glob(glob_str)
        if not files:
            raise FileNotFoundError('No CSV files found in %s' % (kwargs['path']))

        cursor = connection.cursor()
        self.stdout.write("clearing existing data..")
        cursor.execute("TRUNCATE TABLE %s;" % (self.table_name))

        self.stdout.write("importing from files..")
        for f in files:
            self.stdout.write(f)
            fp = open(f, 'r')
            cursor.copy_expert("""
                COPY %s (
                uprn, ctry_flag, cty, ced, lad, ward, parish, hlthau, ctry,
                rgn, pcon, eer, ttwa, nuts, park, oa11, lsoa11, msoa11,
                wz11, ccg, bua11, buasd11, ruc11, oac11, lep1, lep2, pfa, imd)
                FROM STDIN (FORMAT CSV, DELIMITER ',', QUOTE '"', HEADER);
            """ % (self.table_name), fp)
        self.stdout.write("...done")



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('uk_geo_utils', '0002_onspd'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='onspd',
            name='id',
        ),
        migrations.AlterField(
            model_name='onspd',
            name='pcds',
            field=models.CharField(primary_key=True, max_length=8, blank=True, serialize=False),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import django.contrib.gis.db.models.fields


class Migration(migrations.Migration):

    dependencies = [
        ('uk_geo_utils', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Onspd',
            fields=[
                ('id', models.AutoField(auto_created=True, verbose_name='ID', primary_key=True, serialize=False)),
                ('pcd', models.CharField(blank=True, max_length=7)),
                ('pcd2', models.CharField(blank=True, max_length=8)),
                ('pcds', models.CharField(blank=True, max_length=8)),
                ('dointr', models.CharField(blank=True, max_length=6)),
                ('doterm', models.CharField(blank=True, max_length=6)),
                ('oscty', models.CharField(blank=True, max_length=9)),
                ('oslaua', models.CharField(blank=True, max_length=9)),
                ('osward', models.CharField(blank=True, max_length=9)),
                ('usertype', models.CharField(blank=True, max_length=1)),
                ('oseast1m', models.CharField(blank=True, max_length=6)),
                ('osnrth1m', models.CharField(blank=True, max_length=7)),
                ('osgrdind', models.CharField(blank=True, max_length=1)),
                ('oshlthau', models.CharField(blank=True, max_length=9)),
                ('hro', models.CharField(blank=True, max_length=9)),
                ('ctry', models.CharField(blank=True, max_length=9)),
                ('gor', models.CharField(blank=True, max_length=9)),
                ('streg', models.CharField(blank=True, max_length=1)),
                ('pcon', models.CharField(blank=True, max_length=9)),
                ('eer', models.CharField(blank=True, max_length=9)),
                ('teclec', models.CharField(blank=True, max_length=9)),
                ('ttwa', models.CharField(blank=True, max_length=9)),
                ('pct', models.CharField(blank=True, max_length=9)),
                ('nuts', models.CharField(blank=True, max_length=10)),
                ('psed', models.CharField(blank=True, max_length=8)),
                ('cened', models.CharField(blank=True, max_length=6)),
                ('edind', models.CharField(blank=True, max_length=1)),
                ('oshaprev', models.CharField(blank=True, max_length=3)),
                ('lea', models.CharField(blank=True, max_length=3)),
                ('oldha', models.CharField(blank=True, max_length=3)),
                ('wardc91', models.CharField(blank=True, max_length=6)),
                ('wardo91', models.CharField(blank=True, max_length=6)),
                ('ward98', models.CharField(blank=True, max_length=6)),
                ('statsward', models.CharField(blank=True, max_length=6)),
                ('oa01', models.CharField(blank=True, max_length=10)),
                ('casward', models.CharField(blank=True, max_length=6)),
                ('park', models.CharField(blank=True, max_length=9)),
                ('lsoa01', models.CharField(blank=True, max_length=9)),
                ('msoa01', models.CharField(blank=True, max_length=9)),
                ('ur01ind', models.CharField(blank=True, max_length=1)),
                ('oac01', models.CharField(blank=True, max_length=3)),
                ('oldpct', models.CharField(blank=True, max_length=5)),
                ('oa11', models.CharField(blank=True, max_length=9)),
                ('lsoa11', models.CharField(blank=True, max_length=9)),
                ('msoa11', models.CharField(blank=True, max_length=9)),
                ('parish', models.CharField(blank=True, max_length=9)),
                ('wz11', models.CharField(blank=True, max_length=9)),
                ('ccg', models.CharField(blank=True, max_length=9)),
                ('bua11', models.CharField(blank=True, max_length=9)),
                ('buasd11', models.CharField(blank=True, max_length=9)),
                ('ru11ind', models.CharField(blank=True, max_length=2)),
                ('oac11', models.CharField(blank=True, max_length=3)),
                ('lat', models.CharField(blank=True, max_length=10)),
                ('long', models.CharField(blank=True, max_length=10)),
                ('lep1', models.CharField(blank=True, max_length=9)),
                ('lep2', models.CharField(blank=True, max_length=9)),
                ('pfa', models.CharField(blank=True, max_length=9)),
                ('imd', models.CharField(blank=True, max_length=5)),
                ('location', django.contrib.gis.db.models.fields.PointField(null=True, blank=True, srid=4326)),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



import os
import glob
from django.db import connection
from django.core.management.base import BaseCommand
from uk_geo_utils.helpers import get_onspd_model


class Command(BaseCommand):
    """
    To import ONSPD, grab the latest release:
    https://ons.maps.arcgis.com/home/search.html?t=content&q=ONS%20Postcode%20Directory
    and run
    python manage.py import_onspd /path/to/data
    """

    def add_arguments(self, parser):
        parser.add_argument(
            'path',
            help='Path to the directory containing the ONSPD CSVs'
        )

    def handle(self, *args, **kwargs):
        self.table_name = get_onspd_model()._meta.db_table

        glob_str = os.path.join(kwargs['path'], "*.csv")
        files = glob.glob(glob_str)
        if not files:
            raise FileNotFoundError('No CSV files found in %s' % (kwargs['path']))

        cursor = connection.cursor()
        self.stdout.write("clearing existing data..")
        cursor.execute("TRUNCATE TABLE %s;" % (self.table_name))

        self.stdout.write("importing from files..")
        for f in files:
            self.stdout.write(f)
            fp = open(f, 'r')
            cursor.copy_expert("""
                COPY %s (
                pcd, pcd2, pcds, dointr, doterm, oscty, ced, oslaua, osward,
                parish, usertype, oseast1m, osnrth1m, osgrdind, oshlthau,
                nhser, ctry, rgn, streg, pcon, eer, teclec, ttwa, pct, nuts,
                statsward, oa01, casward, park, lsoa01, msoa01, ur01ind,
                oac01, oa11, lsoa11, msoa11, wz11, ccg, bua11, buasd11,
                ru11ind, oac11, lat, long, lep1, lep2, pfa, imd, calncv, stp
                ) FROM STDIN (FORMAT CSV, DELIMITER ',', quote '"', HEADER);
            """ % (self.table_name), fp)

        # turn text lng/lat into a Point() field
        cursor.execute("""
            UPDATE %s SET location=CASE
                WHEN ("long"='0.000000' AND lat='99.999999')
                THEN NULL
                ELSE ST_GeomFromText('POINT(' || "long" || ' ' || lat || ')',4326)
            END
        """ % (self.table_name))

        self.stdout.write("...done")



import os
from setuptools import find_packages, setup


# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

def _get_description():
    try:
        path = os.path.join(os.path.dirname(__file__), 'README.md')
        with open(path, encoding='utf-8') as f:
            return f.read()
    except IOError:
        return ''


setup(
    name='uk_geo_utils',
    version='0.4.0',
    author="chris48s",
    license="MIT",
    url="https://github.com/DemocracyClub/uk-geo-utils",
    packages=find_packages(),
    include_package_data=True,
    description='Django app for working with OS Addressbase, ONSUD and ONSPD',
    long_description=_get_description(),
    long_description_content_type="text/markdown",
    install_requires=[
        'Django>=1.11,<2.0',
        'psycopg2-binary',
    ],
    extras_require={
        'development': [
            'python-coveralls',
            'mkdocs',
        ]
    },
    classifiers=[
        'Framework :: Django',
        'Framework :: Django :: 1.11',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
)
